@@ -7,11 +7,10 @@ use Carp 'carp';
 use v5.10.1;
 
 our @CARP_NOT;
-our $VERSION = '0.15';
-
-use overload '""' => sub { $_[0]->to_string }, fallback => 1;
+our $VERSION = '0.16';
 
 our $DEFAULT_KEY_SIZE = 512;
+our $MAX_GEN_ROUNDS = 100;
 
 # Maximum number of tests for random prime generation = 100
 # Range of valid key sizes = 512 - 4096
@@ -24,7 +23,7 @@ our $DEFAULT_KEY_SIZE = 512;
 # TODO: Improve tests for _bitsize, b64url_encode, b64url_decode,
 #       hex2b64url, b64url2hex
 
-
+use overload '""' => sub { $_[0]->to_string }, fallback => 1;
 use Digest::SHA qw/sha256 sha256_hex/;
 use MIME::Base64 qw(decode_base64 encode_base64);
 
@@ -198,7 +197,9 @@ sub generate {
   my $psize = int( $size / 2 );
 
   my $n;
-  my $m = 100; # Maximum number of rounds
+
+  # Maximum number of rounds
+  my $m = $MAX_GEN_ROUNDS;
 
   my ($p, $q);
 
@@ -356,16 +357,25 @@ sub verify {
       $message,
       $encoded_message) =  @_;
 
-  # Delete whitespace and padding
-  $encoded_message =~ tr{=\t-\x0d }{}d;
-
+  # Missing parameters
   unless ($encoded_message && $message) {
     carp 'No signature or message given';
     return;
   };
 
-  return unless $self->n;
+  # Delete whitespace and padding
+  $encoded_message =~ tr{=\t-\x0d }{}d;
 
+  # Invalid message
+  unless ($encoded_message) {
+    carp 'No signature given';
+    return;
+  };
+
+  # No modulus
+  # return unless $self->n;
+
+  # Verify message
   _verify_emsa_pkcs1_v1_5(
     $self,
     $message,
@@ -379,17 +389,14 @@ sub verify {
 sub to_string {
   my $self = shift;
 
-  my $n = $self->n;
-  return '' unless $n;
+  # return '' unless $n; # Shouldn't be possible
 
-  my $e = $self->e;
+  # Convert modulus and exponent and add to component array
+  my @array = ('RSA', _hex_to_b64url($self->n), _hex_to_b64url($self->e));
 
-  # Convert modulus and exponent
-  $_ = _hex_to_b64url($_) for ($n, $e);
-
-  my @array = ('RSA', $n, $e);
-
-  push(@array, _hex_to_b64url($self->d)) if $_[0] && $self->d;
+  if ($_[0] && $self->d) {
+    push(@array, _hex_to_b64url($self->d));
+  };
 
   # Specification is not clear about $mkey =~ s/=+//g;
   join('.', @array);
@@ -432,7 +439,7 @@ sub b64url_decode {
 
 # Get octet length of n
 sub _emLen {
-  return 0 unless $_[0]->n;
+  # return 0 unless $_[0]->n;
   ($_[0]->[4] // ($_[0]->[4] = _octet_len( $_[0]->n )));
 };
 
@@ -604,7 +611,7 @@ sub _octet_len {
 
 # Returns the bitlength of the integer
 sub _bitsize {
-  my $int = Math::BigInt->new( shift );
+  my $int = Math::BigInt->new( $_[0] );
   return 0 unless $int;
   # Trim leading '0b'
   length( $int->as_bin ) - 2;
@@ -618,9 +625,8 @@ sub _b64url_to_hex {
   #         MagicSignatures/SignatureAlgRsaSha256.pm
 
   # Decode and convert b64url encoded hex number
-  Math::BigInt->new(
-    '0x' . unpack( 'H*', b64url_decode( shift ) )
-  );
+  my $b64 = b64url_decode( shift ) or return;
+  Math::BigInt->new('0x' . unpack( 'H*', $b64));
 };
 
 
@@ -746,6 +752,7 @@ Accepts the attributes C<size> and C<e>.
 In case no C<size> attribute is given, the default key size
 for generation is 512 bits, which is also the minimum size.
 The maximum size is 4096 bits.
+Random prime trials are limited to 100 rounds.
 
 
 =head2 sign
